@@ -15,6 +15,7 @@ from django.db.models.signals import post_save
 from django.core.cache import cache
 from django.dispatch import receiver
 from django.core.urlresolvers import reverse
+from django.core.files.storage import get_storage_class
 
 from .widgets import RichText
 from pizza.middleware import PIZZA_SITES_KEY, PIZZA_DEFAULT_SITE_KEY
@@ -26,6 +27,23 @@ PIZZA_FILE_DIR = getattr(settings, 'PIZZA_FILE_DIR', 'pizza/files/%Y-%m')
 PIZZA_IMAGE_DIR = getattr(settings, 'PIZZA_IMAGE_DIR', 'pizza/images/%Y-%m')
 PIZZA_TEMPLATES = getattr(settings, 'PIZZA_TEMPLATES', {})
 
+def gen_path (instance, filename, base):
+  sclass = get_storage_class()
+  storage = sclass()
+  
+  while 1:
+    path = datetime.datetime.now().strftime(base + '/%d%H%M%S%f_') + filename
+    if not storage.exists(path):
+      break
+      
+  return path
+  
+def file_path (instance, filename):
+  return gen_path(instance, filename, PIZZA_FILE_DIR)
+  
+def image_path (instance, filename):
+  return gen_path(instance, filename, PIZZA_IMAGE_DIR)
+  
 EDITORTYPES = (
   ('rich', 'Rich Text'),
   ('plain', 'Plain Text'),
@@ -106,7 +124,7 @@ class Blurb (models.Model):
   
 class File (ViewFileMixin, models.Model):
   title = models.CharField(max_length=255)
-  file = models.FileField(upload_to=PIZZA_FILE_DIR)
+  file = models.FileField(upload_to=file_path)
   
   @staticmethod
   def autocomplete_search_fields():
@@ -120,7 +138,7 @@ class File (ViewFileMixin, models.Model):
     
 class Image (ViewFileMixin, models.Model):
   title = models.CharField(max_length=255)
-  file = models.ImageField(upload_to=PIZZA_IMAGE_DIR)
+  file = models.ImageField(upload_to=image_path)
   
   caption = models.CharField(max_length=255, blank=True, null=True)
   caption_url = models.CharField('Caption URL', max_length=255, blank=True, null=True)
@@ -269,6 +287,16 @@ class ModelForm (forms.ModelForm):
     instance.cleaned_data = self.cleaned_data
     return instance
     
+class CleanPublish (object):
+  def clean_publish (self):
+    data = self.cleaned_data['publish']
+    if data:
+      now = datetime.datetime.now()
+      if data < now:
+        raise forms.ValidationError("Publish must be in the future")
+        
+    return data
+    
 class Page (SitesMixin, models.Model):
   url = models.CharField('URL', max_length=255, help_text='Examples: / = HomePage, /some_page, /some_page/sub_page')
   tpl = models.CharField("Template", choices=TEMPLATES, max_length=255)
@@ -358,7 +386,7 @@ class Page (SitesMixin, models.Model):
         'desc': forms.CharField(max_length=255, label='Description', required=False, widget=forms.TextInput(attrs={'class': 'vTextField'})),
         'publish': forms.DateTimeField(required=False, widget=AdminSplitDateTime),
         
-        'Meta': Meta
+        'Meta': Meta,
       }
       
       regions = PIZZA_TEMPLATES[self.tpl]['regions']
@@ -379,7 +407,10 @@ class Page (SitesMixin, models.Model):
     if regions and inline:
       fields['DELETE'] = forms.BooleanField(label='Delete', required=False)
       
-    return type('AdminForm', (ModelForm,), fields)
+    if regions and inline:
+      return type('AdminForm', (ModelForm,), fields)
+      
+    return type('AdminForm', (CleanPublish, ModelForm), fields)
     
   def inlines (self, version):
     classes = []
@@ -439,10 +470,18 @@ class Page (SitesMixin, models.Model):
   
   def View_Published (self):
     edit_url = reverse('kitchen_sink:admin_editon')
-    goto = urllib.quote('http://' + self.sites.all()[0].domain + self.url + '?preview=1')
-    return '<a href="%s" target="_blank">View</a>&nbsp; -&nbsp;<a href="%s?goto=%s" target="_blank">Edit on Site</a>' % (self.url, edit_url, goto)
+    goto = urllib.quote('//' + self.domain() + self.url + '?preview=1')
+    data = {'url': self.url, 'edit': edit_url, 'goto': goto, 'domain': self.domain()}
+    return '<a href="//%(domain)s%(url)s" target="_blank">View</a>&nbsp; -&nbsp;<a href="%(edit)s?goto=%(goto)s" target="_blank">Edit on Site</a>' % data
     
   View_Published.allow_tags = True
+  
+  @cached_method
+  def domain (self):
+    if self.sites.all().count() > 0:
+      return self.sites.all()[0].domain
+      
+    return ''
     
 class Redirect (SitesMixin, models.Model):
   url = models.CharField('URL', max_length=255, help_text='Examples: /, /some_page, /some_page/sub_page')
